@@ -950,19 +950,7 @@ install.ffmpeg <- function(...) install.FFmpeg(...)
 #' }
 system.PATH <- function() strsplit(shell("echo %PATH% ", intern= TRUE), ";")[[1]]
 
-#' @title Checks if some .exe is available in on the Windows machine search PATH
-#' @description Checks the existence of an .exe extention in the search path for executable files
-#' @param exe_file a character with the name of the 
-#' @return A boolean vector indication the existence of each program on the system.
-#' @examples
-#' \dontrun{
-#' is.exe.installed(c("zip.exe", "Rgui.exe", "blablabla")) # [1]  TRUE  TRUE FALSE
-#' is.exe.installed("7z") 
-#' }
-is.exe.installed <- function(exe_file) {
-   all_installed_exe <- list.files(system.PATH())
-   exe_file %in% all_installed_exe
-}
+
  
 # ' @title Extends the current path with more possible softwares
 # ' @description Useful for adding new softwares to the current search path
@@ -1067,7 +1055,8 @@ source.https <- function(URL,..., remove_r_file = T) {
 #' @export
 #' @description  require2 load add-on packages by passing it to \link{require}.  However, if the package is not available on the system, it will first install it (through \link{install.packages}), and only then try to load it again.
 #' 
-#' @param package A character of the name of a package.
+#' @param package A character of the name of a package (can also be without quotes).
+#' @param ask Should the user be asked to install the require packaged, in case it is missing? (default is TRUE)
 #' @param ... not used
 #' 
 #' @return  returns (invisibly) a logical indicating whether the required package is available.
@@ -1075,137 +1064,47 @@ source.https <- function(URL,..., remove_r_file = T) {
 #' \dontrun{
 #' a= require2("devtools")
 #' a
-#' a= require2("geonames")
+#' a= require2(geonames)
 #' a
 #' }
-require2 <- function(package, ...) {
-   if(!suppressWarnings(require(package=package, character.only = TRUE))) install.packages(pkgs=package)
+require2 <- function(package, ask= TRUE, ...) {
+   package <- as.character(substitute(package))
+   if(!suppressWarnings(require(package=package, character.only = TRUE))) {
+      install_package <- ask.user.yn.question(paste("Package ",package, " is not installed. Should it be installed?"))
+      if(install_package) install.packages(pkgs=package)
+   }
    require(package=package, character.only = TRUE)
 }
 
 
-# Escaping “@” in Roxygen2 Style Documentation
-# http://stackoverflow.com/questions/8809004/escaping-in-roxygen2-style-documentation
 
 
-#' @title Measures the speed of downloading from different CRAN mirrors
+
+
+
+
+#' @title Restart RGui from RGui
 #' @export
-#' @description Estimates the speed of each CRAN mirror by measuring the time it takes to download the NEWS file.
+#' @description Start a new RGui session and then quites the current one.
 #' 
-#' @author Barry Rowlingson <b.rowlingson@@lancaster.ac.uk>
-#' 
-#' @param ms - the output of getCRANmirrors.  Defaults to using all of the mirrors.
-#' @param ... not in use
-#' 
-#' 
-#' @details
-#' It works by downloading the latest NEWS file (288 Kbytes at the moment, so not huge) 
-#' from each of the mirror sites in the CRAN mirrors list. 
-#' If you want to test it on a subset then call getCRANmirrors yourself and subset it somehow.
-#' 
-#' It runs on the full CRAN list and while desiginig this package I've yet to find a 
-#' timeout or error so I'm not sure what will happen if download.file
-#' fails. It retuns a data frame like you get from getCRANmirrors but
-#' with an extra 't' column giving the elapsed time to get the NEWS file.
-#' 
-#' CAVEATS: if your network has any local caching then these results
-#' will be wrong, since your computer will probably be getting the
-#' locally cached NEWS file and not the one on the server. Especially if
-#' you run it twice. Oh, I should have put cacheOK=FALSE in the
-#' download.file - but even that might get overruled somewhere. Also,
-#' sites may have good days and bad days, good minutes and bad minutes,
-#' your network may be congested on a short-term basis, etc etc.
-#' 
-#' There may also be a difference in reliability, which would not so easily be measured by an individual user.
-#' 
-#' Later that year, Barry also wrote Cranography. See: \url{https://stat.ethz.ch/pipermail/r-help/2009-July/206489.html}, \url{http://www.maths.lancs.ac.uk/~rowlings/R/Cranography/}.
-#' 
-#' @return a data.frame with details on mirror sites and the time it took to download their NEWS file.
-#' 
-#' @source \url{https://stat.ethz.ch/pipermail/r-help/2009-July/206430.html}
+#' This is a Windows only function.
+#' @param ... passed to q()
 #' 
+#' @return  q(...)
 #' @examples
 #' \dontrun{
-#' # this can take some time
-#' x <- cranometer() 
-#' 
-#' time_order <- order(x$t)
-#' 
-#' # a quick overview of the fastest mirrors
-#' head(x[time_order,c(1:4, 9)], 20)
-#' 
-#' # a dotchart of the fastest mirrors
-#' with(x[rev(time_order),],
-#'  dotchart(t, labels =Name,
-#'  cex = .5, xlab = "Timing of CRAN mirror")
-#'  )
-#'
-#'# tail(geonames_df)
-#'# tail(x)
-#'require(plyr)
-#'ss <- !(x$Name == "0-Cloud")
-#'gvis_df <- ddply(x[ss,], .(CountryCode), function(xx) {
-#'   ss <- which.min(xx$t) 
-#'   if(length(ss) == 0) ss <- 1
-#'   data.frame(time = xx$t[ss], name = xx$Name[ss] )
-#'})
-#'gvis_df <- gvis_df[!is.na(gvis_df$time), ]
-#'
-#'require2("googleVis")
-#'Geo<-gvisGeoMap(gvis_df,
-#'               locationvar = "CountryCode",
-#'                numvar="time",
-#'                hovervar = "name",
-#'                options=list(
-#'                             colors='[0xA5EF63, 
-#'                              0xFFB581, 0xFF8747]')
-#'               )
-#'# Display chart
-#'plot(Geo) 
+#' restart_RGui()
 #' }
-cranometer <- function(ms = getCRANmirrors(all = FALSE, local.only = FALSE),...){   
-   dest = tempfile()
-   
-   nms = dim(ms)[1]
-   ms$t = rep(NA,nms)
-   for(i in 1:nms){
-      m = ms[i,]
-      url = paste(m$URL,"/src/base/NEWS",sep="")
-      t = try(system.time(download.file(url,dest),gcFirst=TRUE))
-      if(file.exists(dest)){
-         file.remove(dest)
-         ms$t[i]=t['elapsed']
-      }else{
-         ms$t[i]=NA
-      }
-   }
-   
-   ms$t <- as.numeric(ms$t)
-   
-   return(ms)
+restart_RGui <- function(...) {
+	# start a new RGui
+	# .Last <- function() 
+   if(!is.windows()) stop("This function only works on Windows OS")
+	if(!is.Rgui()) stop("This function only works when running it from Rgui.exe")
+	shell(file.path(R.home("bin"),"Rgui.exe"), wait = FALSE)
+	# close this one:
+	q(...)
 }
 
-## ----
-# # a geomap of mirrors around the world, and how fast each one is
-# require2("geonames")
-# this can take some time
-# find the geo-locations for all of the CRAN mirrors based on http://www.geonames.org/export/geonames-search.html
-# geonames_df <- NULL
-# for(i in 1:nrow(x)) {
-#    tmp_geo <- with(x[i,], GNsearch(name=City, country= CountryCode))[1,]
-#    # if we have no columns, it probably means that GNsearch couldn't find that city name, in which case, we shorten the name of the city and search again.
-#    if(ncol(tmp_geo) == 0) {
-#       tmp_city <- paste(tail(strsplit(x[i,"City"], " ")[[1]], -1), collapse = " ")
-#       tmp_geo <- with(x[i,], GNsearch(name=tmp_city, country= CountryCode))[1,]      
-#    }
-#    if(ncol(tmp_geo) == 0) tmp_geo <- NA # if we still can't find anything, we should turn this to NA so that we would still add a row (though an empy one) to the data.frame
-#       
-#    geonames_df <- rbind(geonames_df,tmp_geo)            
-# }
-# LatLong <- with(geonames_df, paste(lat, ":", lng, sep = ""))
-# gvis_df <- data.frame(LatLong, time = x$t, name = x$Name)
-
-
 
 
 
@@ -1279,4 +1178,3 @@ installr <- function(use_GUI = TRUE, ...) {
 
 
 
-
